@@ -103,6 +103,9 @@ option_strings = {
     'pam_cert_db_path' : _('Path to certificate database with PKCS#11 modules.'),
     'p11_child_timeout' : _('How many seconds will pam_sss wait for p11_child to finish'),
     'pam_app_services' : _('Which PAM services are permitted to contact application domains'),
+    'pam_p11_allowed_services' : _('Allowed services for using smartcards'),
+    'p11_wait_for_card_timeout' : _('Additional timeout to wait for a card if requested'),
+    'p11_uri' : _('PKCS#11 URI to restrict the selection of devices for Smartcard authentication'),
 
     # [sudo]
     'sudo_timed' : _('Whether to evaluate the time-based attributes in sudo rules'),
@@ -116,6 +119,8 @@ option_strings = {
     'ssh_hash_known_hosts': _('Whether to hash host names and addresses in the known_hosts file'),
     'ssh_known_hosts_timeout': _('How many seconds to keep a host in the known_hosts file after its host keys were requested'),
     'ca_db': _('Path to storage of trusted CA certificates'),
+    'ssh_use_certificate_keys': _('Allow to generate ssh-keys from certificates'),
+    'ssh_use_certificate_matching_rules': _('Use the following matching rules to filter the certificates for ssh-key generation'),
 
     # [pac]
     'allowed_uids': _('List of UIDs or user names allowed to access the PAC responder'),
@@ -163,12 +168,13 @@ option_strings = {
     'max_id' : _('Maximum user ID'),
     'enumerate' : _('Enable enumerating all users/groups'),
     'cache_credentials' : _('Cache credentials for offline login'),
-    'store_legacy_passwords' : _('Store password hashes'),
     'use_fully_qualified_names' : _('Display users/groups in fully-qualified form'),
     'ignore_group_members' : _('Don\'t include group members in group lookups'),
     'entry_cache_timeout' : _('Entry cache timeout length (seconds)'),
     'lookup_family_order' : _('Restrict or prefer a specific address family when performing DNS lookups'),
     'account_cache_expiration' : _('How long to keep cached entries after last successful login (days)'),
+    'dns_resolver_server_timeout' : _('How long should SSSD talk to single DNS server before trying next server (miliseconds)'),
+    'dns_resolver_op_timeout' : _('How long should keep trying to resolve single DNS query (seconds)'),
     'dns_resolver_timeout' : _('How long to wait for replies from DNS when resolving servers (seconds)'),
     'dns_discovery_domain' : _('The domain part of service discovery DNS query'),
     'override_gid' : _('Override GID value from the identity provider with this value'),
@@ -280,6 +286,7 @@ option_strings = {
     'ldap_backup_uri' : _('ldap_backup_uri, The URI of the LDAP server'),
     'ldap_search_base' : _('The default base DN'),
     'ldap_schema' : _('The Schema Type in use on the LDAP server, rfc2307'),
+    'ldap_pwmodify_mode' : _('Mode used to change user password'),
     'ldap_default_bind_dn' : _('The default bind DN'),
     'ldap_default_authtok_type' : _('The type of the authentication token of the default bind DN'),
     'ldap_default_authtok' : _('The authentication token of the default bind DN'),
@@ -405,8 +412,6 @@ option_strings = {
     'ldap_idmap_default_domain_sid' : _('SID of the default domain for ID-mapping'),
     'ldap_idmap_helper_table_size' : _('Number of secondary slices'),
 
-    'ldap_groups_use_matching_rule_in_chain' : _('Use LDAP_MATCHING_RULE_IN_CHAIN for group lookups'),
-    'ldap_initgroups_use_matching_rule_in_chain' : _('Use LDAP_MATCHING_RULE_IN_CHAIN for initgroup lookups'),
     'ldap_use_tokengroups' : _('Whether to use Token-Groups'),
     'ldap_min_id' : _('Set lower boundary for allowed IDs from the LDAP server'),
     'ldap_max_id' : _('Set upper boundary for allowed IDs from the LDAP server'),
@@ -437,6 +442,7 @@ option_strings = {
     'ldap_sudo_include_netgroups' : _('Whether to include rules that contains netgroup in host attribute'),
     'ldap_sudo_include_regexp' : _('Whether to include rules that contains regular expression in host attribute'),
     'ldap_sudorule_object_class' : _('Object class for sudo rules'),
+    'ldap_sudorule_object_class_attr' : _('Name of attribute that is used as object class for sudo rules'),
     'ldap_sudorule_name' : _('Sudo rule name'),
     'ldap_sudorule_command' : _('Sudo rule command attribute'),
     'ldap_sudorule_host' : _('Sudo rule host attribute'),
@@ -500,16 +506,14 @@ class SSSDConfigSchema(SSSDChangeConf):
             schemaplugindir = '/usr/share/sssd/sssd.api.d'
 
         try:
-            #Read the primary config file
-            fd = open(schemafile, 'r')
-            self.readfp(fd)
-            fd.close()
+            # Read the primary config file
+            with open(schemafile, 'r') as fd:
+                self.readfp(fd)
             # Read in the provider files
             for file in filter(lambda f: re.search(r'^sssd-.*\.conf$', f),
                                          os.listdir(schemaplugindir)):
-                fd = open(schemaplugindir+ "/" + file)
-                self.readfp(fd)
-                fd.close()
+                with open(schemaplugindir+ "/" + file) as fd:
+                    self.readfp(fd)
         except IOError:
             raise
         except SyntaxError: # can be raised with readfp
@@ -1452,14 +1456,12 @@ class SSSDConfig(SSSDChangeConf):
             #TODO: get this from a global setting
             configfile = '/etc/sssd/sssd.conf'
         # open will raise an IOError if it fails
-        fd = open(configfile, 'r')
+        with open(configfile, 'r') as fd:
+            try:
+                self.readfp(fd)
+            except:
+                raise ParsingError
 
-        try:
-            self.readfp(fd)
-        except:
-            raise ParsingError
-
-        fd.close()
         self.configfile = configfile
         self.initialized = True
 
@@ -1523,10 +1525,9 @@ class SSSDConfig(SSSDChangeConf):
 
         # open() will raise IOError if it fails
         old_umask = os.umask(0o177)
-        of = open(outputfile, "wb")
-        output = self.dump(self.opts).encode('utf-8')
-        of.write(output)
-        of.close()
+        with open(outputfile, "w") as of:
+            output = self.dump(self.opts)
+            of.write(output)
         os.umask(old_umask)
 
     def list_active_services(self):
@@ -1919,6 +1920,35 @@ class SSSDConfig(SSSDChangeConf):
         providers = [ (x['name'],x['value']) for x in self.strip_comments_empty(self.options('domain/%s' % name))
                      if x['name'].rfind('_provider') > 0]
 
+        # Handle the default value for providers if those weren't set in the configuration file
+        default_providers = {
+            'id_provider': None,
+            'sudo_provider': 'id_provider',
+            'auth_provider': 'id_provider',
+            'chpass_provider': 'auth_provider',
+            'autofs_provider': 'id_provider',
+            'selinux_provider': 'id_provider',
+            'subdomains_provider': 'id_provider',
+            'session_provider': 'id_provider',
+            'hostid_provider': 'id_provider',
+        }
+
+        configured_providers = domain.list_providers()
+        providers_list = [x[0] for x in providers]
+
+        if 'access_provider' not in providers_list:
+            providers.append(('access_provider', 'permit'))
+
+        for provider, default_provider in default_providers.items():
+            if provider in providers_list:
+                continue
+
+            if default_provider in providers_list:
+                default_provider_value = providers[providers_list.index(default_provider)]
+                if default_provider_value[1] in configured_providers.keys():
+                    if provider[:provider.rfind('_provider')] in configured_providers[default_provider_value[1]]:
+                        providers.append((provider, default_provider_value[1]))
+
         for (option, value) in providers:
             try:
                 domain.set_option(option, value)
